package de.kif.backend.route

import de.kif.backend.Configuration
import de.kif.backend.repository.PostRepository
import de.kif.backend.repository.RoomRepository
import de.kif.backend.repository.ScheduleRepository
import de.kif.backend.view.respondMain
import de.kif.common.model.Schedule
import io.ktor.routing.Route
import io.ktor.routing.get
import kotlinx.css.CSSBuilder
import kotlinx.css.Color
import kotlinx.html.*
import java.util.*
import kotlin.math.max
import kotlin.math.min

fun Route.board() {
    get("/brett") {
        val scheduleList = ScheduleRepository.all().map {
            it to it.getAbsoluteStartTime() * 60
        }.sortedBy { it.second }

        val referenceTime = Configuration.Schedule.referenceDate.time / 1000
        val now = referenceTime - (Date().time / 1000)

        val refDate = Configuration.Schedule.referenceDate
        val todayDate = Date()

        val refDay = refDate.time / (1000 * 60 * 60 * 24)
        val todayDay = todayDate.time / (1000 * 60 * 60 * 24)
        val day = (todayDay - refDay).toInt()

        val list = ScheduleRepository.getByDay(day)
        val rooms = RoomRepository.all()
        val schedules = list.groupBy { it.room }.mapValues { (_, it) ->
            it.associateBy {
                it.time
            }
        }

        var max = 0
        var min = 24 * 60
        for (s in list) {
            max = max(max, s.time + s.workGroup.length)
            min = min(min, s.time)
        }

        if (min > max) {
            val h1 = max
            max = min
            min = h1
        }

        if (true) {
            min = min(min, 0)
            max = max(max, 24 * 60)
        }

        min = (min / 60 - 1) * 60
        max = (max / 60 + 2) * 60

        respondMain(true, true) { theme ->
            content {
                div("board") {
                    div("board-header") {
                        div("board-running") {
                            for ((schedule, time) in scheduleList) {
                                div("board-schedule") {
                                    attributes["data-id"] = schedule.id.toString()

                                    div("board-schedule-color") {
                                        span {
                                            attributes["style"] = CSSBuilder().apply {
                                                val c = schedule.workGroup.track?.color
                                                if (c != null) {
                                                    backgroundColor = Color(c.toString())
                                                }
                                            }.toString()
                                        }
                                    }

                                    div("board-schedule-time") {
                                        attributes["data-time"] = time.toString()
                                        attributes["data-duration"] = schedule.workGroup.length.toString()


                                        val startTime = (time % MINUTES_OF_DAY).let {
                                            if (it < 0) it + MINUTES_OF_DAY else it
                                        }
                                        val sm = (startTime % 60).toString().padStart(2, '0')
                                        val sh = (startTime / 60).toString().padStart(2, '0')
                                        val startTimeString = "$sh:$sm"

                                        val endTime = ((time + schedule.workGroup.length) % MINUTES_OF_DAY).let {
                                            if (it < 0) it + MINUTES_OF_DAY else it
                                        }
                                        val em = (endTime % 60).toString().padStart(2, '0')
                                        val eh = (endTime / 60).toString().padStart(2, '0')
                                        val endTimeString = "$eh:$em"

                                        +"$startTimeString - $endTimeString"
                                    }

                                    div("board-schedule-name") {
                                        +schedule.workGroup.name
                                    }

                                    div("board-schedule-room") {
                                        +schedule.room.name
                                    }
                                }
                            }
                        }
                        div("board-logo") {
                            img("KIF 47.0", "/static/images/logo.svg")
                            div("board-header-date") {
                            }
                        }
                    }
                    div("board-content") {
                        div("board-calendar calendar") {
                            div("calendar-table") {
                                renderCalendar(
                                    CalendarOrientation.ROOM_TO_TIME,
                                    day,
                                    min,
                                    max,
                                    rooms,
                                    schedules
                                )
                            }
                        }
                        div("board-twitter") {
                            unsafe {
                                raw("""
                                <a
                                    class="twitter-timeline"
                                    href="${Configuration.Twitter.timeline}"
                                    data-chrome="transparent noheader nofooter"
                                    data-theme="${if (theme.dark) "dark" else "light"}"
                                    data-link-color="${theme.primaryColor}"
                                    data-cards="hidden"
                                    data-lang="de"
                                    data-dnt="true"
                                >Twitter wall</a>
                                <script async src="https://platform.twitter.com/widgets.js" charset="utf-8"></script>
                            """.trimIndent())
                            }
                        }
                    }
                }
            }
        }
    }
}
