package de.kif.backend.route

import de.kif.backend.repository.RoomRepository
import de.kif.backend.repository.ScheduleRepository
import de.kif.backend.view.respondMain
import de.kif.common.model.Room
import de.kif.common.model.Schedule
import io.ktor.routing.Route
import io.ktor.routing.get
import kotlinx.html.div
import kotlin.math.max
import kotlin.math.min

data class WallData(
    val number: Int,
    val schedules: Map<Room, Map<Int, Schedule>>,
    val max: Int,
    val min: Int
)

suspend fun genWallData(day: Int): WallData {
    val list = ScheduleRepository.getByDay(day)
    val schedules = RoomRepository.all().associateWith { emptyMap<Int, Schedule>() } + list.groupBy { it.room }.mapValues { (_, it) ->
        it.associateBy {
            it.time
        }
    }

    var max = 0
    var min = 24 * 60
    for (s in list) {
        max = max(max, s.time + s.workGroup.length)
        min = min(min, s.time)
    }

    if (min > max) {
        val h1 = max
        max = min
        min = h1
    }

    min = (min / 60 - 1) * 60
    max = (max / 60 + 2) * 60

    return WallData(day, schedules, max, min)
}

fun Route.wall() {
    get("/wand") {

        val days = (0..2).map { genWallData(it) }

        val min = days.map { it.min }.min() ?: days.first().min
        val max = days.map { it.max }.max() ?: days.first().max

        respondMain(true, true) {
            content {
                div("wall") {
                    for (day in days) {
                        div("wall-box") {
                            div("wall-calendar calendar") {
                                div("calendar-table") {
                                    renderCalendar(
                                        CalendarOrientation.TIME_TO_ROOM,
                                        day.number,
                                        min,
                                        max,
                                        day.schedules.keys.toList().sortedBy { it.id },
                                        day.schedules
                                    )
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
